@@ -21,6 +21,7 @@ from limits.storage import MemoryStorage, Storage, storage_from_string
 from limits.strategies import STRATEGIES, RateLimiter
 from werkzeug.http import http_date, parse_date
 
+from ._compat import request_context
 from .constants import MAX_BACKEND_CHECKS, ConfigVars, ExemptionScope, HeaderNames
 from .errors import RateLimitExceeded
 from .manager import LimitManager
@@ -411,26 +412,24 @@ class Limiter:
         if not hasattr(app, "extensions"):
             app.extensions = {}  # pragma: no cover
 
-        if not app.extensions.get("limiter"):
+        if self not in app.extensions.setdefault("limiter", set()):
             if self._auto_check:
                 app.before_request(self._check_request_limit)
 
-            # The use of partial is simply to satisfy mypy
             app.after_request(partial(Limiter.__inject_headers, self))
             app.teardown_request(self.__release_context)
-
-        app.extensions["limiter"] = self
+        app.extensions["limiter"].add(self)
         self.initialized = True
 
     @property
     def context(self) -> LimiterContext:
-        ctx = flask._request_ctx_stack.top
-        assert ctx
+        ctx = request_context()
         if not hasattr(ctx, "_limiter_request_context"):
-            ctx._limiter_request_context = defaultdict(LimiterContext)
-        return cast(Dict[str, LimiterContext], ctx._limiter_request_context)[
-            self._key_prefix
-        ]
+            ctx._limiter_request_context = defaultdict(LimiterContext)  # type: ignore
+        return cast(
+            Dict[str, LimiterContext],
+            ctx._limiter_request_context,  # type: ignore
+        )[self._key_prefix]
 
     def limit(
         self,
@@ -786,7 +785,6 @@ class Limiter:
     ) -> flask.wrappers.Response:
         self.__check_conditional_deductions(response)
         header_limit = self.current_limit
-
         if (
             self.enabled
             and self._headers_enabled
